/*******************************************************************************
         Copyright (c) 2020, MaxLinear, Inc.
         Copyright 2016 - 2020 Intel Corporation
         Copyright 2015 - 2016 Lantiq Beteiligungs-GmbH & Co. KG
         Copyright 2009 - 2014 Lantiq Deutschland GmbH
         Copyright 2007 - 2008 Infineon Technologies AG

  For licensing information, see the file 'LICENSE' in the root folder of
  this software module.

*******************************************************************************/

#ifndef __ATM_TC_CYC__
#define __ATM_TC_CYC__

/*Get current cycle count*/
inline unsigned int cpu_get_cycles(void)
{
        unsigned int count;

        __asm__ __volatile__(
        "       .set push\n"
        "       .set mips32r2\n"
        "       rdhwr   %0, $2\n"
        "       .set pop\n"
        : "=r" (count));

        return count;
}

/*Cycles count register update resolution*/
inline u32 get_counter_resolution(void)
{
        u32 res;

        __asm__ __volatile__(
                ".set   push\n"
                ".set   mips32r2\n"
                "rdhwr  %0, $3\n"
                ".set pop\n"
                : "=&r" (res)
                : /* no input */
                : "memory");

        return res;
}

/*Returns freq in Hz*/
inline u32 cpu_freq(void)
{
	u32 start, end, freq = 0;
	u32 sec = 1;

	start = cpu_get_cycles();
	udelay(sec*1000000);
	end = cpu_get_cycles();

	freq = end-start;
	freq = freq/sec;

	return freq;
}

#define cyc2ms(x)			(atmtc_u64_div((x*1000), __cpu_freq, NULL))
#define ms2cyc(x)			(x*atmtc_u64_div(__cpu_freq, 1000, NULL))
#define CORE_COUNT			2
#define UPDATE_INTERVAL_MS		1000
#define U64				u32
#define FRACTION_DIGITS 		2
//#define COUNTERS_NORMALISE_PER_SEC(x)	((x*1000)/UPDATE_INTERVAL_MS)

static u8 core_map[4] = {0 , 'x', 1, 'x'};
static U64 tx_cyc[CORE_COUNT], rx_cyc[CORE_COUNT];	//updated by datapath
static U64 tx_cyc_l[CORE_COUNT], rx_cyc_l[CORE_COUNT];	//updated by proc
//static U64 tx_iter[CORE_COUNT], rx_iter[CORE_COUNT];	//updated by datapath
//static U64 tx_iter_l[CORE_COUNT], rx_iter_l[CORE_COUNT];//updated by proc
static spinlock_t lck[CORE_COUNT];
static u32 __cpu_freq = 0;

#define ATMTC_CYCLE_START(x, cid)		\
{	x  = cpu_get_cycles(); 			\
	spin_lock(&lck[cid]);			\
	spin_unlock(&lck[cid]);			\
}

#define ATMTC_CYCLE_STOP(s, txrx, cid)		\
{	u32 cycl = cpu_get_cycles();		\
	spin_lock(&lck[cid]);			\
	txrx##_cyc[cid] += (cycl - s); 		\
	/*txrx##_iter[cid]++;*/			\
	spin_unlock(&lck[cid]);			\
}

inline U64 atmtc_u64_div_rem(U64 dev, U64 div, U64 *rem)
{
#ifdef U64_DIV_BRUTE_FORCE
	u64 cnt = 0;
	if (!dev) {
		if (rem)
			*rem = 0;
		return 0;
	}
	while (dev > div) {
		cnt++;
		dev -= div;
	}
	if (rem)
		*rem = dev;
	return cnt;
#else
	if (rem)
	       *rem = dev%div;
	return (dev/div);
#endif
}

inline U64 atmtc_u64_div(U64 dev, U64 div, U64 *fract)
{
        int i;
        U64 cnt = 0, remainder = 0;
        U64 fdigit = 0, fraction = 0;
	cnt = atmtc_u64_div_rem(dev, div, &remainder);

	if (remainder == 0) {
		if (fract)
			*fract = 0;
		return cnt;
	}

	if (fract) {
		for (i=0; i<FRACTION_DIGITS; i++) {
	                remainder *= 10;
	                fdigit = atmtc_u64_div_rem(remainder, div, &remainder);
	                if (i) {
	                        fraction *= 10;
	                }
	                fraction += fdigit;
	        }
	        *fract = fraction;
	}

        return cnt;
}

enum cpu_ld_mtx_idx1 {
	TX=0,
	RX=1,
	TXRX=2,
	IDX1_MAX,
};

enum cpu_ld_mtx_idx2 {
	DEC=0,
	FRA=1,
	IDX2_MAX,
};

#endif
