#!/bin/bash

mod_reload=0
DEV_ID=0
OLD_PPE_FW=0
powersave=0
ATM_NUM_CONN=0
d=$( date | sed 's/ /_/g' )
TEST_LOG_FILE=vrx_mode_test_$d.log
ping_test_var=0
ping_test_iter=0
basemode=""
nextmode=""
prev_test=""
curr_test=""
only_test=0

ATM_IFACE_DEL() {
	brctl delif br-lan nas0
	ifconfig nas0 down
	kill -2 `pidof br2684ctl`
	sleep 3
}

ATM_IFACE_CFG() {
	br2684ctl -b -c 0 -e 0 -a 0.35
	sleep 2
	ifconfig nas0 up
	sleep 2
	brctl addif br-lan nas0
	arp -s 192.168.1.3 00:01:02:03:04:05
}

ATM_PING_TEST() {
	PING_TEST "nas0" $1
}

PTM_PING_TEST() {
	PING_TEST "ptm0" $1
}

ATM_SL_INIT() {
	if [ $mod_reload == 1 ]; then
		insmod vrx518 && insmod vrx518_tc.ko loopback_en=1 powersaving=$powersave;
	fi;
	sleep 2;
	echo load atm $DEV_ID > /proc/driver/vrx518/tc_switch
	sleep 20
	if [ OLD_PPE_FW == 1 ]; then 
		echo w 2020 0x880000 > /proc/driver/vrx518/atm$DEV_ID/mem;
	fi
	sleep 2
	echo 0 yes 1500 > /proc/driver/vrx518/showtime
	sleep 2
	ATM_IFACE_CFG
}

ATM_EXIT() {
	ATM_IFACE_DEL
	echo unload > /proc/driver/vrx518/tc_switch
	if [ $mod_reload == 1 ]; then
		rmmod vrx518_tc
		rmmod vrx518
	fi
	sleep 1
}

ATM_BOND_INIT() {
	if [ $mod_reload == 1 ]; then
		insmod vrx518 && insmod vrx518_tc.ko loopback_en=1 powersaving=$powersave;
	fi
	sleep 2
	echo load atm_bond > /proc/driver/vrx518/tc_switch
	sleep 20
	echo 0 yes 1500 > /proc/driver/vrx518/showtime
	sleep 2
	echo 1 yes 1500 > /proc/driver/vrx518/showtime
	sleep 2
	ATM_IFACE_CFG
}

PTM_IFACE_CFG() {
	arp -s 192.168.1.3 00:01:02:03:04:05
	brctl addif br-lan ptm0_wan3
}

PTM_IFACE_DEL() {
	brctl delif br-lan ptm0_wan3
}

PTM_SL_INIT() {
	if [ $mod_reload == 1 ]; then
		insmod vrx518 && insmod vrx518_tc.ko loopback_en=1;
	fi
	sleep 2
	echo load ptm $DEV_ID > /proc/driver/vrx518/tc_switch
	sleep 20
	echo $DEV_ID yes 1500 > /proc/driver/vrx518/showtime
	sleep 2
	PTM_IFACE_CFG
}

PTM_BOND_INIT() {
	if [ $mod_reload == 1 ]; then
		insmod vrx518 && insmod vrx518_tc.ko loopback_en=1;
	fi
	sleep 2
	echo load ptm_bond > /proc/driver/vrx518/tc_switch
	sleep 20
	echo 0 yes 1500 > /proc/driver/vrx518/showtime
	sleep 2
	echo 1 yes 1500 > /proc/driver/vrx518/showtime
	sleep 2
	PTM_IFACE_CFG
}

PTM_EXIT() {
	PTM_IFACE_DEL
	echo unload > /proc/driver/vrx518/tc_switch
	if [ $mod_reload == 1 ]; then
		rmmod vrx518_tc
		rmmod vrx518
	fi
	sleep 1
}

PING_TEST() {
	TX_BEFORE=$(ifconfig $1 | grep -o -E 'TX packets:[0-9]+' | grep -o -E '[0-9]+')
	RX_BEFORE=$(ifconfig $1 | grep -o -E 'RX packets:[0-9]+' | grep -o -E '[0-9]+')
	if ((ping_test_iter == 0)); then
		failed_test_name=""
		ping_test_var=0
		prev_test=""
		curr_test=""
	elif ((ping_test_iter == 1));then
		prev_test=$curr_test;
	fi

	curr_test=$2
	
	ping 192.168.1.3 -c 2 -p aa

	RX_CNT=$(ifconfig $1 | grep -o -E 'RX packets:[0-9]+' | grep -o -E '[0-9]+')
	TX_CNT=$(ifconfig $1 | grep -o -E 'TX packets:[0-9]+' | grep -o -E '[0-9]+')
	((TX_CNT -= TX_BEFORE))
	((RX_CNT -= RX_BEFORE))

	if ((RX_CNT == 0)); then
		failed_test_name=$curr_test
	else
		((ping_test_var++))
	fi

	if ((only_test == 1)); then
		if ((ping_test_var == 1)); then
			echo "$curr_test: VRX518_TC Test Passed" | tee -a $TEST_LOG_FILE
		else
			echo "$curr_test: VRX518_TC Test Failed" | tee -a $TEST_LOG_FILE
		fi
	else
		if ((ping_test_iter == 1 && ping_test_var == 2)); then
			echo "$prev_test => $curr_test: VRX518_TC Test Passed" | tee -a $TEST_LOG_FILE
		elif ((ping_test_iter == 1 && ping_test_var == 1)); then
			echo "$prev_test => $curr_test: VRX518_TC Test Failed $failed_test_name" | tee -a $TEST_LOG_FILE
		elif ((ping_test_iter == 1 && ping_test_var == 0)); then
			echo "$prev_test => $curr_test: VRX518_TC Both are Failed" | tee -a $TEST_LOG_FILE
		fi
	fi

	((ping_test_iter++));
	((ping_test_iter%=2));
}

ATM_SL_TEST() {
	ATM_SL_INIT
	ATM_PING_TEST "ATM_SL"
	ATM_EXIT
}

PTM_SL_TEST() {
	PTM_SL_INIT
	PTM_PING_TEST "PTM_SL"
	PTM_EXIT
}

ATM_BOND_TEST() {
	ATM_BOND_INIT
	ATM_PING_TEST "ATM_BOND"
	ATM_EXIT
}

PTM_BOND_TEST() {
	PTM_BOND_INIT
	PTM_PING_TEST "PTM_BOND"
	PTM_EXIT
}

modecheck() {
	if [ $1 != "atmsl" -a $1 != "ptmsl" ]; then
		if [ $1 != "atmbond" -a $1 != "ptmbond" ]; then
			usage
			exit 133
		fi
	fi
}

TEST() {
	if [ $1 == "atmsl" ]; then
		ATM_SL_TEST
	elif [ $1 == "ptmsl" ]; then
		PTM_SL_TEST
	elif [ $1 == "atmbond" ]; then
		ATM_BOND_TEST
	elif [ $1 == "ptmbond" ]; then
		PTM_BOND_TEST
	else
		usage
		exit 143
	fi
}

TEST_PAIR() {
	modecheck $1
	modecheck $2
	TEST $1
	TEST $2
}

TEST_ONE_MODE_FULL() {
	bmode=$1
	if [ $bmode == "atmsl" ]; then
		TEST_PAIR $bmode "atmsl"
		TEST_PAIR $bmode "ptmsl"
		TEST_PAIR $bmode "atmbond"
		TEST_PAIR $bmode "ptmbond"
	elif [ $bmode == "ptmsl" ]; then
		TEST_PAIR $bmode "ptmsl"
		TEST_PAIR $bmode "atmsl"
		TEST_PAIR $bmode "atmbond"
		TEST_PAIR $bmode "ptmbond"
	elif [ $bmode == "atmbond" ]; then
		TEST_PAIR $bmode "atmbond"
		TEST_PAIR $bmode "atmsl"
		TEST_PAIR $bmode "ptmsl"
		TEST_PAIR $bmode "ptmbond"
	elif [ $bmode == "ptmbond" ]; then
		TEST_PAIR $bmode "ptmbond"
		TEST_PAIR $bmode "atmsl"
		TEST_PAIR $bmode "ptmsl"
		TEST_PAIR $bmode "atmbond"
	else
		usage
		exit 123
	fi

}

TEST_ALL() {
	TEST_ONE_MODE_FULL "atmsl"
	TEST_ONE_MODE_FULL "ptmsl"
	TEST_ONE_MODE_FULL "atmbond"
	TEST_ONE_MODE_FULL "ptmbond"
}

usage() {
	echo "## USAGE: $0 all/set_lbkenv/clr_lbkenv/upd_ppefw"
	echo "## USAGE: $0 only <basemode>"
	echo "## USAGE: $0 mode <basemode>"
	echo "## USAGE: $0 pair <basemode> <nextmode>"
	echo "## modes: atmsl ptmsl atmbond ptmbond"
}

load_modules() {
	if [ $mod_reload == 0 ]; then
		lsmod | grep vrx518 > /dev/null
		if [ $? -ne 0 ]; then
			insmod vrx518 && insmod vrx518_tc.ko loopback_en=1 powersaving=$powersave;
		fi
	fi;
}

############## execution starts here

if [ $# -eq 3 ]; then
	if [ $1 != "pair" ]; then
		usage
		exit 444
	fi
	load_modules
	TEST_PAIR $2 $3
elif [ $# -eq 2 ]; then
	if [ $1 != "mode" -a $1 != "only" ]; then
		usage
		exit 555
	fi
	load_modules
	if [ $1 == "mode" ]; then
		TEST_ONE_MODE_FULL $2
	elif [ $1 == "only" ]; then
		only_test=1
		TEST $2
	fi
elif [ $# -eq 1 ]; then
	if [ $1 == "all" ]; then
		load_modules
		TEST_ALL
	elif [ $1 == "set_lbkenv" ]; then
		echo "" > /etc/modules.d/26-vrx518_tc_drv
		echo "" > /etc/modules.d/27-vrx518_ep
		echo "" > /etc/modules.d/44-dsl-cpe-mei-vrx
		echo "" > /etc/modules.d/45-dsl-cpe-api-vrx
		reboot
	elif [ $1 == "clr_lbkenv" ]; then
		rm -rf /overlay/upper/etc/modules.d
		reboot
	elif [ $1 == "upd_ppefw" ]; then
		cd /lib/firmware
		tftp -g 192.168.1.2 -r ppe_fw.bin
		md5sum ppe_fw.bin
		cd -
	else
		usage
		exit 333
	fi
else
	usage
	exit 222
fi

touch $TEST_LOG_FILE
cat $TEST_LOG_FILE
